@@ -148,6 +148,7 @@ typedef struct _xmlParserCtxt {
     int                nameMax;       /* Max depth of the parsing stack */
     xmlChar *         *nameTab;       /* array of nodes */
 
+    long               nbChars;       /* number of xmlChar processed */
 } _xmlParserCtxt;
 typedef _xmlParserCtxt xmlParserCtxt;
 typedef xmlParserCtxt *xmlParserCtxtPtr;
@@ -267,10 +268,14 @@ extern int xmlSubstituteEntitiesDefaultValue;
 #include "entities.h"
 #include "xml-error.h"
 
+/**
+ * Cleanup
+ */
+void		xmlCleanupParser	(void);
+
 /**
  * Input functions
  */
-
 int		xmlParserInputRead	(xmlParserInputPtr in,
 					 int len);
 int		xmlParserInputGrow	(xmlParserInputPtr in,
